# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest


class TestAAZContentBuilder(unittest.TestCase):

    @staticmethod
    def _define_args_schema():
        from azure.cli.core.aaz._arg import AAZArgumentsSchema, AAZStrArg, AAZObjectArg, AAZListArg, AAZDictArg, \
            AAZFreeFormDictArg, AAZAnyTypeArg
        _schema = AAZArgumentsSchema()

        # str
        _schema.name = AAZStrArg(nullable=True)

        # object
        _schema.obj = AAZObjectArg(nullable=True)
        _schema.obj.prop1 = AAZStrArg()

        _schema.h_obj = AAZObjectArg(nullable=True)
        _schema.h_obj.prop1 = AAZStrArg()

        # dict<str, str>
        _schema.tags = AAZDictArg(nullable=True)
        _schema.tags.Element = AAZStrArg()

        # dict<str, str>
        _schema.h_tags = AAZDictArg(nullable=True)
        _schema.h_tags.Element = AAZStrArg()

        # list<str>
        _schema.permissions = AAZListArg(nullable=True)
        _schema.permissions.Element = AAZStrArg()

        _schema.h_permissions = AAZListArg(nullable=True)
        _schema.h_permissions.Element = AAZStrArg()

        # list<object>
        _schema.subnets = AAZListArg()
        _schema.subnets.Element = AAZObjectArg(nullable=True)
        _schema.subnets.Element.name = AAZStrArg()

        # list<list<str>>
        _schema.adds = AAZListArg()
        _schema.adds.Element = AAZListArg(nullable=True)
        _schema.adds.Element.Element = AAZStrArg()

        # dict<str, object>
        _schema.domains = AAZDictArg()
        _schema.domains.Element = AAZObjectArg(nullable=True)
        _schema.domains.Element.name = AAZStrArg()

        # dict<str, dict<str, str>>
        _schema.conns = AAZDictArg()
        _schema.conns.Element = AAZDictArg(nullable=True)
        _schema.conns.Element.Element = AAZStrArg()

        _schema.actions = AAZListArg()
        element = _schema.actions.Element = AAZObjectArg()
        element.name = AAZStrArg()

        element.action_a = AAZObjectArg()
        element.action_a.web = AAZStrArg()
        element.action_a.author = AAZStrArg()

        element.action_b = AAZObjectArg()
        element.action_b.machine = AAZStrArg()
        element.action_b.country = AAZStrArg()

        element.action_c = AAZStrArg()
        element.action_d = AAZStrArg()

        _schema.freeform = AAZFreeFormDictArg(nullable=True)
        _schema.h_freeform = AAZFreeFormDictArg(nullable=True)

        _schema.actions_any = AAZListArg()
        _schema.actions_any.Element = AAZAnyTypeArg(nullable=True)
        _schema.any1 = AAZAnyTypeArg()
        _schema.any2 = AAZAnyTypeArg()
        _schema.any3 = AAZAnyTypeArg()
        _schema.any4 = AAZAnyTypeArg()
        _schema.any5 = AAZAnyTypeArg(nullable=True)
        _schema.any6 = AAZAnyTypeArg(nullable=True)
        _schema.h_any = AAZAnyTypeArg(nullable=True)

        return _schema

    def test_aaz_content_builder_for_create(self):
        from azure.cli.core.aaz._content_builder import AAZContentBuilder
        from azure.cli.core.aaz._arg_browser import AAZArgBrowser
        from azure.cli.core.aaz._field_type import AAZStrType, AAZObjectType, AAZListType, AAZDictType, AAZFreeFormDictType, AAZAnyType

        _args_schema = self._define_args_schema()
        arg_value = _args_schema(data={
            "name": "a",
            "obj": {
                "prop1": "pp1",
            },
            "tags": {
                "tag_a": "a",
                "tag_b": "b",
            },
            "permissions": [
                "read", "write",
            ],
            "subnets": [
                {
                    "name": "net1",
                },
                {
                    "name": "net2",
                }
            ],
            "adds": [
                ["0"],
                ["0", "1", "2"],
                ["2", "3"],
            ],
            "domains": {
                "a": {
                    "name": "0",
                },
                "b": {
                    "name": "1",
                },
                "c": {
                    "name": "2",
                }
            },
            "conns": {
                "a": {
                    "a1": "0",
                    "a2": "1",
                },
                "b": {
                    "b1": "3",
                    "b2": "6",
                }
            },
            "freeform": {
                "a": 1,
                "b": "string",
                "c": None,
                "d": True,
                "e": [1, "str", False],
                "f": {"a": 1, "b": "str"},
            },
            "actions_any": [
                False,
                2,
                2.5,
                "str",
                ["5", "6", "7"],
                {
                    "a": 3,
                    "b": "string",
                    "c": None,
                },
            ],
            "any1": 2,
            "any2": 2.5,
            "any3": "str",
            "any4": False,
            "any5": ["5", "6", "7"],
            "any6": {
                "a": 3,
                "b": "string",
                "c": None,
            },
        })
        arg_data = arg_value.to_serialized_data(keep_undefined_in_list=True)
        schema = AAZObjectType()
        _value = AAZObjectType._ValueCls(
            schema=schema,
            data=schema.process_data(None)
        )

        _builder = AAZContentBuilder(
            values=[_value],
            args=[AAZArgBrowser(arg_value=arg_value, arg_data=arg_data)]
        )

        _builder.set_prop('name', AAZStrType, '.name')

        self.assertTrue(
            _value.to_serialized_data()['name'] == 'a'
        )

        _builder.set_prop('obj', AAZObjectType, '.obj')
        _builder.set_prop('hideObj', AAZObjectType, '.h_obj')

        obj = _builder.get('.obj')
        if obj is not None:
            obj.set_prop('prop1', AAZStrType, '.prop1')

        hide_obj = _builder.get('.hideObj')
        if hide_obj:
            hide_obj.set_prop('prop1', AAZStrType, '.prop1')

        self.assertEqual(
            _value.to_serialized_data()['obj'], {'prop1': "pp1"}
        )
        self.assertTrue('hideObj' not in _value.to_serialized_data())

        _builder.set_prop('tags', AAZDictType, '.tags')
        _builder.set_prop('hideTags', AAZDictType, '.h_tags')

        tags = _builder.get('.tags')
        if tags is not None:
            tags.set_elements(AAZStrType, '.')

        hide_tags = _builder.get('.hideTags')
        if hide_tags:
            hide_tags.set_elements(AAZStrType, '.')

        self.assertEqual(_value.to_serialized_data()['tags'], {'tag_a': 'a', 'tag_b': 'b'})
        self.assertTrue('hideTags' not in _value.to_serialized_data())

        _builder.set_prop('permissions', AAZListType, '.permissions')
        _builder.set_prop('hidePermissions', AAZListType, '.h_permissions')

        permissions = _builder.get('.permissions')
        if permissions:
            permissions.set_elements(AAZStrType, '.')

        hide_permissions = _builder.get('.hidePermissions')
        if hide_permissions:
            hide_permissions.set_elements(AAZStrType, '.')

        self.assertTrue(_value.to_serialized_data()['permissions'] == ['read', 'write'])
        self.assertTrue('hidePermissions' not in _value.to_serialized_data())

        _builder.set_prop('properties', AAZObjectType)

        self.assertTrue('properties' not in _value.to_serialized_data())

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('subnets', AAZListType, '.subnets')

        subnets = _builder.get('.properties.subnets')
        if subnets:
            subnets.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.subnets[]')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['subnets'], [
            {'name': 'net1'},
            {'name': 'net2'}
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('adds', AAZListType, '.adds')

        adds = _builder.get('.properties.adds')
        if adds:
            adds.set_elements(AAZListType, '.')

        elements = _builder.get('.properties.adds[]')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.adds[][]')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        self.assertEqual(_value.to_serialized_data()['properties']['adds'], [
            [{'name': '0'}],
            [{'name': '0'}, {'name': '1'}, {'name': '2'}],
            [{'name': '2'}, {'name': '3'}]
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('domains', AAZDictType, '.domains')

        domains = _builder.get('.properties.domains')
        if domains:
            domains.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.domains{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['domains'], {
            'a': {'name': '0'},
            'b': {'name': '1'},
            'c': {'name': '2'}
        })

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('conns', AAZDictType, '.conns')

        conns = _builder.get('.properties.conns')
        if conns:
            conns.set_elements(AAZDictType, '.')

        elements = _builder.get('.properties.conns{}')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.conns{}{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        self.assertTrue(_value.to_serialized_data()['properties']['conns'] == {
            'a': {
                'a1': {'name': '0'},
                'a2': {'name': '1'}
            },
            'b': {
                'b1': {'name': '3'},
                'b2': {'name': '6'},
            }
        })

        _builder.set_prop('freeForm', AAZFreeFormDictType, '.freeform')
        _builder.set_prop('hideFreeForm', AAZFreeFormDictType, '.h_freeform')

        freeform = _builder.get('.freeForm')
        if freeform:
            freeform.set_anytype_elements('.')

        h_freeform = _builder.get('.hideFreeForm')
        if h_freeform:
            h_freeform.set_anytype_elements('.')

        _builder.set_prop('actionsAny', AAZListType, '.actions_any')
        actionsAny = _builder.get('.actionsAny')
        if actionsAny:
            actionsAny.set_elements(AAZAnyType, '.')
        
        _builder.set_prop('any1', AAZAnyType, '.any1')
        _builder.set_prop('any2', AAZAnyType, '.any2')
        _builder.set_prop('any3', AAZAnyType, '.any3')
        _builder.set_prop('any4', AAZAnyType, '.any4')
        _builder.set_prop('any5', AAZAnyType, '.any5')
        _builder.set_prop('any6', AAZAnyType, '.any6')
        _builder.set_prop('hideAny', AAZAnyType, '.h_any')

        _data = _value.to_serialized_data()
        print(_data)

        self.assertEqual(_data,
                         {'name': 'a',
                          'obj': {'prop1': 'pp1'},
                          'tags': {'tag_a': 'a', 'tag_b': 'b'},
                          'permissions': ['read', 'write'],
                          'properties': {'subnets': [{'name': 'net1'}, {'name': 'net2'}],
                                         'adds': [[{'name': '0'}],
                                                  [{'name': '0'}, {'name': '1'},
                                                   {'name': '2'}],
                                                  [{'name': '2'}, {'name': '3'}]],
                                         'domains': {'a': {'name': '0'},
                                                     'b': {'name': '1'},
                                                     'c': {'name': '2'}}, 'conns': {
                                  'a': {'a1': {'name': '0'}, 'a2': {'name': '1'}},
                                  'b': {'b1': {'name': '3'}, 'b2': {'name': '6'}}}},
                          "freeForm": {
                              "a": 1,
                              "b": "string",
                              "c": None,
                              "d": True,
                              "e": [1, "str", False],
                              "f": {"a": 1, "b": "str"},
                          },
                          "actionsAny": [
                              False,
                              2,
                              2.5,
                              "str",
                              ["5", "6", "7"],
                              {
                                  "a": 3,
                                  "b": "string",
                                  "c": None,
                              },
                          ],
                          "any1": 2,
                          "any2": 2.5,
                          "any3": "str",
                          "any4": False,
                          "any5": ["5", "6", "7"],
                          "any6": {
                              "a": 3,
                              "b": "string",
                              "c": None,
                          }
                          })

    def _define_instance_value(self):
        from azure.cli.core.aaz._field_type import AAZStrType, AAZObjectType, AAZListType, AAZDictType, AAZFreeFormDictType, AAZAnyType
        from azure.cli.core.aaz._field_value import AAZObject

        _schema = AAZObjectType()

        _schema.name = AAZStrType()
        _schema.obj = AAZObjectType()
        _schema.obj.prop1 = AAZStrType()
        _schema.hideObj = AAZObjectType(nullable=True)
        _schema.hideObj.prop1 = AAZStrType()

        _schema.tags = AAZDictType()
        _schema.tags.Element = AAZStrType()
        _schema.hideTags = AAZDictType(nullable=True)
        _schema.hideTags.Element = AAZStrType()
        _schema.permissions = AAZListType()
        _schema.permissions.Element = AAZStrType()
        _schema.hidePermissions = AAZListType(nullable=True)
        _schema.hidePermissions.Element = AAZStrType()
        _schema.properties = AAZObjectType()
        _schema.properties.subnets = AAZListType()
        _schema.properties.subnets.Element = AAZObjectType(nullable=True)
        _schema.properties.subnets.Element.name = AAZStrType()
        _schema.properties.adds = AAZListType()
        _schema.properties.adds.Element = AAZListType()
        _schema.properties.adds.Element.Element = AAZObjectType()
        _schema.properties.adds.Element.Element.name = AAZStrType()
        _schema.properties.domains = AAZDictType()
        _schema.properties.domains.Element = AAZObjectType(nullable=True)
        _schema.properties.domains.Element.name = AAZStrType()
        _schema.properties.conns = AAZDictType()
        _schema.properties.conns.Element = AAZDictType()
        _schema.properties.conns.Element.Element = AAZObjectType()
        _schema.properties.conns.Element.Element.name = AAZStrType()
        _schema.freeForm = AAZFreeFormDictType()
        _schema.hideFreeForm = AAZFreeFormDictType(nullable=True)
        _schema.actionsAny = AAZListType()
        _schema.actionsAny.Element = AAZAnyType(nullable=True)
        _schema.any1 = AAZAnyType()
        _schema.any2 = AAZAnyType()
        _schema.any3 = AAZAnyType()
        _schema.any4 = AAZAnyType()
        _schema.any5 = AAZAnyType()
        _schema.any6 = AAZAnyType(nullable=True)
        _schema.hideAny = AAZAnyType(nullable=True)

        value = AAZObject(_schema, data=_schema.process_data({
            'name': 'a',
            'obj': {'prop1': 'pp1'},
            'hideObj': {'prop1': 'hp1'},
            'tags': {'tag_a': 'a', 'tag_b': 'b'},
            'hideTags': {'h_tag_a': 'a', 'h_tag_b': 'b'},
            'permissions': ['read', 'write'],
            'hidePermissions': ['read', 'write'],
            'properties': {
                'subnets': [{'name': 'net1'}, {'name': 'net2'}],
                'adds': [[{'name': '0'}], [{'name': '0'}, {'name': '1'}, {'name': '2'}],
                         [{'name': '2'}, {'name': '3'}]],
                'domains': {'a': {'name': '0'}, 'b': {'name': '1'}, 'c': {'name': '2'}},
                'conns': {'a': {'a1': {'name': '0'}, 'a2': {'name': '1'}}, 'b': {'b1': {'name': '3'}}}
            },
            "freeForm": {"a": 1, "b": "string", "c": None},
            "hideFreeForm": {"h_a": 1, "h_b": "string", "h_c": None},
            "actionsAny": [
                False,
                0,
                0.5,
                "str",
                None,
                ["1", "2", "3"],
                {
                    "a": 1,
                    "b": "string",
                    "c": None,
                },
            ],
            "any1": 1,
            "any2": 0.5,
            "any3": "str",
            "any4": {"a": 1, "b": "string", "c": None},
            "any5": False,
            "any6": ["1", "2", "3"],
            "hideAny": {"a": 666, "b": "string", "c": None},
        }))

        return value

    def test_aaz_content_builder_for_update(self):
        from azure.cli.core.aaz._content_builder import AAZContentBuilder
        from azure.cli.core.aaz._arg_browser import AAZArgBrowser
        from azure.cli.core.aaz._field_type import AAZStrType, AAZObjectType, AAZListType, AAZDictType, AAZFreeFormDictType, AAZAnyType

        _value = self._define_instance_value()

        _args_schema = self._define_args_schema()
        arg_value = _args_schema(data={})
        arg_value.name = 'b'

        arg_value.obj.prop1 = 'p2'
        arg_value.h_obj.prop1 = 'hh'

        arg_value.tags = {
            'tag_a': '2'
        }

        arg_value.h_tags['h_tag_a'] = '2'

        arg_value.permissions = ["write"]

        arg_value.h_permissions[0] = 'copy'
        arg_value.h_permissions[2] = 'delete'

        arg_value.subnets[0].name = 'net'
        arg_value.subnets[2] = {'name': "net3"}

        arg_value.adds[0][1] = '1'
        arg_value.adds[1] = ['1', '2']
        arg_value.adds[3][0] = '4'

        arg_value.domains['a'].name = 'a'
        arg_value.domains['d'] = {'name': 'd'}

        arg_value.conns['a'] = {'f1': 'f1'}
        arg_value.conns['b']['b1'] = 'b1'
        arg_value.conns['b']['b3'] = "b3"
        arg_value.conns['d'] = {"d": "d"}

        arg_value.freeform = {"b": 123}

        arg_value.actions_any[0] = None

        arg_value.any5 = None
        arg_value.any6 = None


        _builder = AAZContentBuilder(
            values=[_value],
            args=[AAZArgBrowser(
                arg_value=arg_value, arg_data=arg_value.to_serialized_data(keep_undefined_in_list=True))]
        )

        _builder.set_prop('name', AAZStrType, '.name')

        self.assertTrue(_value.to_serialized_data()['name'] == 'b')

        _builder.set_prop('obj', AAZObjectType, '.obj')
        _builder.set_prop('hideObj', AAZObjectType, '.h_obj')

        obj = _builder.get('.obj')
        if obj is not None:
            obj.set_prop('prop1', AAZStrType, '.prop1')

        hide_obj = _builder.get('.hideObj')
        if hide_obj:
            hide_obj.set_prop('prop1', AAZStrType, '.prop1')

        self.assertEqual(
            _value.to_serialized_data()['obj'], {'prop1': "p2"}
        )
        self.assertEqual(
            _value.to_serialized_data()['hideObj'], {'prop1': "hh"}
        )

        _builder.set_prop('tags', AAZDictType, '.tags')
        _builder.set_prop('hideTags', AAZDictType, '.h_tags')

        tags = _builder.get('.tags')
        if tags is not None:
            tags.set_elements(AAZStrType, '.')

        hide_tags = _builder.get('.hideTags')
        if hide_tags:
            hide_tags.set_elements(AAZStrType, '.')

        self.assertTrue(_value.to_serialized_data()['tags'] == {'tag_a': '2'})
        self.assertTrue(_value.to_serialized_data()['hideTags'] == {'h_tag_a': '2', 'h_tag_b': 'b'})

        _builder.set_prop('permissions', AAZListType, '.permissions')
        _builder.set_prop('hidePermissions', AAZListType, '.h_permissions')

        permissions = _builder.get('.permissions')
        if permissions:
            permissions.set_elements(AAZStrType, '.')

        hide_permissions = _builder.get('.hidePermissions')
        if hide_permissions:
            hide_permissions.set_elements(AAZStrType, '.')

        self.assertTrue(_value.to_serialized_data()['permissions'] == ['write'])
        self.assertTrue(_value.to_serialized_data()['hidePermissions'] == ['copy', 'write', 'delete'])

        _builder.set_prop('properties', AAZObjectType)

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('subnets', AAZListType, '.subnets')

        subnets = _builder.get('.properties.subnets')
        if subnets:
            subnets.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.subnets[]')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['subnets'], [
            {'name': 'net'},
            {'name': 'net2'},
            {'name': 'net3'},
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('adds', AAZListType, '.adds')

        adds = _builder.get('.properties.adds')
        if adds:
            adds.set_elements(AAZListType, '.')

        elements = _builder.get('.properties.adds[]')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.adds[][]')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        self.assertEqual(_value.to_serialized_data()['properties']['adds'], [
            [{'name': '0'}, {'name': '1'}],
            [{'name': '1'}, {'name': '2'}],
            [{'name': '2'}, {'name': '3'}],
            [{'name': '4'}]
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('domains', AAZDictType, '.domains')

        domains = _builder.get('.properties.domains')
        if domains:
            domains.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.domains{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['domains'], {
            'a': {'name': 'a'},
            'b': {'name': '1'},
            'c': {'name': '2'},
            'd': {'name': 'd'},
        })

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('conns', AAZDictType, '.conns')

        conns = _builder.get('.properties.conns')
        if conns:
            conns.set_elements(AAZDictType, '.')

        elements = _builder.get('.properties.conns{}')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.conns{}{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        _builder.set_prop('freeForm', AAZFreeFormDictType, '.freeform')
        _builder.set_prop('hideFreeForm', AAZFreeFormDictType, '.h_freeform')

        freeform = _builder.get('.freeForm')
        if freeform:
            freeform.set_anytype_elements('.')

        h_freeform = _builder.get('.hideFreeForm')
        if h_freeform:
            h_freeform.set_anytype_elements('.')
        
        _builder.set_prop('actionsAny', AAZListType, '.actions_any')
        actionsAny = _builder.get('.actionsAny')
        if actionsAny:
            actionsAny.set_elements(AAZAnyType, '.')

        _builder.set_prop('any1', AAZAnyType, '.any1')
        _builder.set_prop('any2', AAZAnyType, '.any2')
        _builder.set_prop('any3', AAZAnyType, '.any3')
        _builder.set_prop('any4', AAZAnyType, '.any4')
        _builder.set_prop('any5', AAZAnyType, '.any5')
        _builder.set_prop('any6', AAZAnyType, '.any6')
        _builder.set_prop('hideAny', AAZAnyType, '.h_any')

        self.assertEqual(_value.to_serialized_data()['properties']['conns'], {
            'a': {
                'f1': {'name': 'f1'},
            },
            'b': {
                'b1': {'name': 'b1'},
                'b3': {'name': 'b3'},
            },
            'd': {
                'd': {'name': 'd'}
            }
        })

        self.assertEqual(_value.to_serialized_data(), {
            'name': 'b',
            'obj': {'prop1': 'p2'},
            'hideObj': {'prop1': 'hh'},
            'tags': {'tag_a': '2'},
            'hideTags': {'h_tag_a': '2', 'h_tag_b': 'b'},
            'permissions': ['write'],
            'hidePermissions': ['copy', 'write', 'delete'],
            'properties': {
                'subnets': [{'name': 'net'}, {'name': 'net2'}, {'name': 'net3'}],
                'adds': [[{'name': '0'}, {'name': '1'}], [{'name': '1'}, {'name': '2'}], [{'name': '2'}, {'name': '3'}],
                         [{'name': '4'}]],
                'domains': {'a': {'name': 'a'}, 'b': {'name': '1'}, 'c': {'name': '2'}, 'd': {'name': 'd'}},
                'conns': {'a': {'f1': {'name': 'f1'}}, 'b': {'b1': {'name': 'b1'}, 'b3': {'name': 'b3'}},
                          'd': {'d': {'name': 'd'}}}
            },
            "freeForm": {"b": 123},
            "hideFreeForm": {"h_a": 1, "h_b": "string", "h_c": None},
            "actionsAny": [
                None,
                0,
                0.5,
                "str",
                None,
                ["1", "2", "3"],
                {
                    "a": 1,
                    "b": "string",
                    "c": None,
                },
            ],
            "any1": 1,
            "any2": 0.5,
            "any3": "str",
            "any4": {"a": 1, "b": "string", "c": None},
            "any6": None,
            "hideAny": {"a": 666, "b": "string", "c": None},
        })

    def test_aaz_content_builder_for_update_with_nullable(self):
        from azure.cli.core.aaz._content_builder import AAZContentBuilder
        from azure.cli.core.aaz._arg_browser import AAZArgBrowser
        from azure.cli.core.aaz._field_type import AAZStrType, AAZObjectType, AAZListType, AAZDictType, AAZUndefined, AAZFreeFormDictType, AAZAnyType

        _value = self._define_instance_value()

        _args_schema = self._define_args_schema()
        arg_value = _args_schema(data={})

        arg_value.name = None
        arg_value.obj = None
        arg_value.h_obj = None
        arg_value.tags = None
        arg_value.h_tags = None
        arg_value.permissions = None
        arg_value.h_permissions = None

        arg_value.subnets[0] = None
        arg_value.domains['a'] = None

        arg_value.adds[0] = None
        arg_value.adds[4] = None

        arg_value.conns['a'] = None
        arg_value.conns['d'] = None

        arg_value.freeform = None
        arg_value.h_freeform = None

        arg_value.actions_any[0] = None
        arg_value.any5 = None
        arg_value.any6 = None
        arg_value.h_any = None

        _builder = AAZContentBuilder(
            values=[_value],
            args=[AAZArgBrowser(
                arg_value=arg_value,
                arg_data=arg_value.to_serialized_data(keep_undefined_in_list=True))
            ]
        )

        _builder.set_prop('name', AAZStrType, '.name')
        self.assertTrue('name' not in _value.to_serialized_data())

        _builder.set_prop('obj', AAZObjectType, '.obj')
        _builder.set_prop('hideObj', AAZObjectType, '.h_obj')

        obj = _builder.get('.obj')
        if obj is not None:
            obj.set_prop('prop1', AAZStrType, '.prop1')

        hide_obj = _builder.get('.hideObj')
        if hide_obj:
            hide_obj.set_prop('prop1', AAZStrType, '.prop1')

        self.assertTrue('obj' not in _value.to_serialized_data())
        self.assertTrue(_value.to_serialized_data()['hideObj'] is None)

        _builder.set_prop('tags', AAZDictType, '.tags')
        _builder.set_prop('hideTags', AAZDictType, '.h_tags')

        tags = _builder.get('.tags')
        if tags is not None:
            tags.set_elements(AAZStrType, '.')

        hide_tags = _builder.get('.hideTags')
        if hide_tags:
            hide_tags.set_elements(AAZStrType, '.')

        self.assertTrue('tags' not in _value.to_serialized_data())
        self.assertTrue(_value.to_serialized_data()['hideTags'] is None)

        _builder.set_prop('permissions', AAZListType, '.permissions')
        _builder.set_prop('hidePermissions', AAZListType, '.h_permissions')

        permissions = _builder.get('.permissions')
        if permissions:
            permissions.set_elements(AAZStrType, '.')

        hide_permissions = _builder.get('.hidePermissions')
        if hide_permissions:
            hide_permissions.set_elements(AAZStrType, '.')

        self.assertTrue('permissions' not in _value.to_serialized_data())
        self.assertTrue(_value.to_serialized_data()['hidePermissions'] is None)

        _builder.set_prop('properties', AAZObjectType)

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('subnets', AAZListType, '.subnets')

        subnets = _builder.get('.properties.subnets')
        if subnets:
            subnets.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.subnets[]')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['subnets'], [
            None,
            {'name': 'net2'},
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('domains', AAZDictType, '.domains')

        domains = _builder.get('.properties.domains')
        if domains:
            domains.set_elements(AAZObjectType, '.')

        elements = _builder.get('.properties.domains{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')

        self.assertEqual(_value.to_serialized_data()['properties']['domains'], {
            'a': None,
            'b': {'name': '1'},
            'c': {'name': '2'}
        })

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('adds', AAZListType, '.adds')

        adds = _builder.get('.properties.adds')
        if adds:
            adds.set_elements(AAZListType, '.')

        elements = _builder.get('.properties.adds[]')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.adds[][]')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        self.assertEqual(_value.to_serialized_data(keep_undefined_in_list=True)['properties']['adds'], [
            AAZUndefined,
            [{'name': '0'}, {'name': '1'}, {'name': '2'}],
            [{'name': '2'}, {'name': '3'}],
            AAZUndefined,
            AAZUndefined,
        ])

        self.assertEqual(_value.to_serialized_data()['properties']['adds'], [
            [{'name': '0'}, {'name': '1'}, {'name': '2'}],
            [{'name': '2'}, {'name': '3'}],
        ])

        properties = _builder.get('.properties')
        if properties:
            properties.set_prop('conns', AAZDictType, '.conns')

        conns = _builder.get('.properties.conns')
        if conns:
            conns.set_elements(AAZDictType, '.')

        elements = _builder.get('.properties.conns{}')
        if elements:
            elements.set_elements(AAZObjectType)

        elements = _builder.get('.properties.conns{}{}')
        if elements:
            elements.set_prop('name', AAZStrType, '.')

        self.assertEqual(_value.to_serialized_data()['properties']['conns'], {
            'b': {'b1': {'name': '3'}}
        })

        _builder.set_prop('freeForm', AAZFreeFormDictType, '.freeform')
        _builder.set_prop('hideFreeForm', AAZFreeFormDictType, '.h_freeform')

        freeform = _builder.get('.freeForm')
        if freeform:
            freeform.set_anytype_elements('.')

        h_freeform = _builder.get('.hideFreeForm')
        if h_freeform:
            h_freeform.set_anytype_elements('.')
        
        _builder.set_prop('actionsAny', AAZListType, '.actions_any')
        actionsAny = _builder.get('.actionsAny')
        if actionsAny:
            actionsAny.set_elements(AAZAnyType, '.')

        _builder.set_prop('any1', AAZAnyType, '.any1')
        _builder.set_prop('any2', AAZAnyType, '.any2')
        _builder.set_prop('any3', AAZAnyType, '.any3')
        _builder.set_prop('any4', AAZAnyType, '.any4')
        _builder.set_prop('any5', AAZAnyType, '.any5')
        _builder.set_prop('any6', AAZAnyType, '.any6')
        _builder.set_prop('hideAny', AAZAnyType, '.h_any')


        self.assertEqual(_value.to_serialized_data(), {
            'hideObj': None, 'hideTags': None, 'hidePermissions': None, "hideFreeForm": None,
            "actionsAny": [
                None,
                0,
                0.5,
                "str",
                None,
                ["1", "2", "3"],
                {
                    "a": 1,
                    "b": "string",
                    "c": None,
                },
            ],
            "any1": 1,
            "any2": 0.5,
            "any3": "str",
            "any4": {"a": 1, "b": "string", "c": None},
            "any6": None,
            "hideAny": None,
            'properties': {
                'subnets': [None, {'name': 'net2'}],
                'adds': [[{'name': '0'}, {'name': '1'}, {'name': '2'}], [{'name': '2'}, {'name': '3'}]],
                'domains': {'a': None, 'b': {'name': '1'}, 'c': {'name': '2'}}, 'conns': {'b': {'b1': {'name': '3'}}}}
        })

    def test_aaz_content_builder_for_polymorphism(self):
        from azure.cli.core.aaz._content_builder import AAZContentBuilder
        from azure.cli.core.aaz._arg_browser import AAZArgBrowser
        from azure.cli.core.aaz._field_type import AAZStrType, AAZObjectType, AAZListType

        _args_schema = self._define_args_schema()
        arg_value = _args_schema(data={
            "name": "a",
            "actions": [
                {
                    "name": "A 1",
                    "action_a": {
                        "web": "Web 1",
                        "author": "Author 1"
                    }
                },
                {
                    "name": "B 1",
                    "action_b": {
                        "machine": "Machine 1",
                        "country": "Country 1",
                    }
                },
                {
                    "name": "A 2",
                    "action_a": {
                        "web": "Web 2",
                        "author": "Author 2"
                    }
                },
                {
                    "name": "B 2",
                    "action_b": {
                        "machine": "Machine 2",
                        "country": "Country 2",
                    }
                },
                {
                    "name": "C 1",
                    "action_c": "Str C"
                }
            ]
        })
        arg_data = arg_value.to_serialized_data(keep_undefined_in_list=True)
        schema = AAZObjectType()
        _value = AAZObjectType._ValueCls(
            schema=schema,
            data=schema.process_data(None)
        )

        _builder = AAZContentBuilder(
            values=[_value],
            args=[AAZArgBrowser(arg_value=arg_value, arg_data=arg_data)]
        )

        _builder.set_prop("name", AAZStrType, '.name')
        self.assertTrue(
            _value.to_serialized_data()['name'] == 'a'
        )

        _builder.set_prop('actions', AAZListType, '.actions')

        actions = _builder.get('.actions')
        if actions:
            actions.set_elements(AAZObjectType, '.')

        elements = _builder.get('.actions[]')
        if elements:
            elements.set_prop('name', AAZStrType, '.name')
            elements.set_const('type', 'A', AAZStrType, '.action_a')
            elements.set_const('type', 'B', AAZStrType, '.action_b')
            elements.set_const('type', 'C', AAZStrType, '.action_c')
            elements.set_const('type', 'D', AAZStrType, '.action_d')
            elements.set_const('default', 'Default', AAZStrType)

            elements.discriminate_by('type', 'A')
            elements.discriminate_by('type', 'B')
            elements.discriminate_by('type', 'C')
            elements.discriminate_by('type', 'D')

        self.assertTrue(_value.to_serialized_data()['actions'] == [
            {'name': 'A 1', 'type': 'A', 'default': 'Default'},
            {'name': 'B 1', 'type': 'B', 'default': 'Default'},
            {'name': 'A 2', 'type': 'A', 'default': 'Default'},
            {'name': 'B 2', 'type': 'B', 'default': 'Default'},
            {'name': 'C 1', 'type': 'C', 'default': 'Default'},
        ])

        disc_a = _builder.get('.actions[]{type:A}')
        if disc_a:
            disc_a.set_prop("web", AAZStrType, '.action_a.web')
            disc_a.set_prop("author", AAZStrType, '.action_a.author')

        disc_b = _builder.get('.actions[]{type:B}')
        if disc_b:
            disc_b.set_prop("machine", AAZStrType, '.action_b.machine')
            disc_b.set_prop("country", AAZStrType, '.action_b.country')

        disc_c = _builder.get('.actions[]{type:C}')
        if disc_c:
            disc_c.set_prop("kind", AAZStrType, '.action_c')

        disc_d = _builder.get('.actions[]{type:D}')
        if disc_d:
            disc_d.set_prop("people", AAZStrType, '.action_d')

        self.assertTrue(_value.to_serialized_data()['actions'] == [
            {'name': 'A 1', 'type': 'A', 'default': 'Default', 'web': 'Web 1', 'author': 'Author 1'},
            {'name': 'B 1', 'type': 'B', 'default': 'Default', 'machine': 'Machine 1', 'country': 'Country 1'},
            {'name': 'A 2', 'type': 'A', 'default': 'Default', 'web': 'Web 2', 'author': 'Author 2'},
            {'name': 'B 2', 'type': 'B', 'default': 'Default', 'machine': 'Machine 2', 'country': 'Country 2'},
            {'name': 'C 1', 'type': 'C', 'default': 'Default', 'kind': 'Str C'}
        ])

        self.assertTrue(_value.actions[0].web == "Web 1")
        element_schema = _value.actions._schema.Element

        self.assertTrue(element_schema.get_attr_name("name") is not None)
        self.assertTrue(element_schema.get_attr_name("type") is not None)

        self.assertTrue(element_schema._discriminator_field_name == "type")

        self.assertTrue(element_schema._discriminators['A'].get_attr_name('web') is not None)
        self.assertTrue(element_schema._discriminators['A'].get_attr_name('author') is not None)
        self.assertTrue(element_schema._discriminators['A'].get_attr_name('machine') is None)
        self.assertTrue(element_schema._discriminators['A'].get_attr_name('country') is None)
        self.assertTrue(element_schema._discriminators['A'].get_attr_name('kind') is None)
        self.assertTrue(element_schema._discriminators['A'].get_attr_name('people') is None)

        self.assertTrue(element_schema._discriminators['B'].get_attr_name('web') is None)
        self.assertTrue(element_schema._discriminators['B'].get_attr_name('author') is None)
        self.assertTrue(element_schema._discriminators['B'].get_attr_name('machine') is not None)
        self.assertTrue(element_schema._discriminators['B'].get_attr_name('country') is not None)
        self.assertTrue(element_schema._discriminators['B'].get_attr_name('kind') is None)
        self.assertTrue(element_schema._discriminators['B'].get_attr_name('people') is None)

        self.assertTrue(element_schema._discriminators['C'].get_attr_name('web') is None)
        self.assertTrue(element_schema._discriminators['C'].get_attr_name('author') is None)
        self.assertTrue(element_schema._discriminators['C'].get_attr_name('machine') is None)
        self.assertTrue(element_schema._discriminators['C'].get_attr_name('country') is None)
        self.assertTrue(element_schema._discriminators['C'].get_attr_name('kind') is not None)
        self.assertTrue(element_schema._discriminators['C'].get_attr_name('people') is None)

        self.assertTrue(element_schema._discriminators['D'].get_attr_name('web') is None)
        self.assertTrue(element_schema._discriminators['D'].get_attr_name('author') is None)
        self.assertTrue(element_schema._discriminators['D'].get_attr_name('machine') is None)
        self.assertTrue(element_schema._discriminators['D'].get_attr_name('country') is None)
        self.assertTrue(element_schema._discriminators['D'].get_attr_name('kind') is None)
        self.assertTrue(element_schema._discriminators['D'].get_attr_name('people') is None)

    def test_split_keys(self):
        from azure.cli.core.aaz._content_builder import AAZContentBuilder
        parts = AAZContentBuilder._split_key(key=".properties.resourceSettings{}.abc[].d")
        self.assertEqual(parts, ['properties', 'resourceSettings', '{}', 'abc', '[]', 'd'])
        parts = AAZContentBuilder._split_key(key=".properties.resourceSettings{resourceType:Microsoft.Network/publicIPAddresses}")
        self.assertEqual(parts, ['properties', 'resourceSettings', '{resourceType:Microsoft.Network/publicIPAddresses}'])
        parts = AAZContentBuilder._split_key(key=".properties.resourceSettings{resourceType:Microsoft.Network.networkInterfaces}{}.ipConfigurations[].loadBalancerNatRules")
        self.assertEqual(parts, ['properties', 'resourceSettings', '{resourceType:Microsoft.Network.networkInterfaces}', '{}', 'ipConfigurations', '[]', 'loadBalancerNatRules'])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.command_modules.monitor.aaz.latest.monitor.diagnostic_settings.categories._show \
    import Show as _DiagnosticSettingsCategoryShow
from azure.cli.command_modules.monitor.operations.diagnostics_settings import (
    create_resource_parameters, update_resource_parameters,
)


class DiagnosticSettingsCategoryShow(_DiagnosticSettingsCategoryShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        arg_schema = super()._build_arguments_schema(*args, **kwargs)
        create_resource_parameters(arg_schema)
        return arg_schema

    def pre_operations(self):
        update_resource_parameters(self.ctx)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-many-lines
# pylint: disable=inconsistent-return-statements
# pylint: disable=protected-access
# pylint: disable=too-many-locals

import argparse


class AlertAddEncryption(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core import CLIError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        keyVaultObject = {}
        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'key-name':
                keyVaultObject["key_name"] = v
            elif k == 'key-vault-uri':
                keyVaultObject["key_vault_uri"] = v
                if keyVaultObject["key_vault_uri"].endswith('/'):
                    keyVaultObject["key_vault_uri"] = keyVaultObject["key_vault_uri"][:-1]
            elif k == 'key-version':
                keyVaultObject["key_version"] = v
            elif k == 'user-assigned-identity':
                keyVaultObject["user_assigned_identity"] = {}
                keyVaultObject["user_assigned_identity"] = v
                if keyVaultObject["user_assigned_identity"].endswith('/'):
                    keyVaultObject["user_assigned_identity"] = keyVaultObject["user_assigned_identity"][:-1]
            else:
                raise InvalidArgumentValueError(
                    "Invalid Argument for:'{}' Only allowed arguments are 'key-name, key-vault-uri, key-version and user-assigned-identity'".format(
                        option_string))

        if (keyVaultObject["key_name"] is None) or (keyVaultObject["key_vault_uri"] is None):
            raise CLIError('key-name and key-vault-uri are mandatory properties')

        if "key_version" not in keyVaultObject:
            keyVaultObject["key_version"] = ''

        if "user_assigned_identity" not in keyVaultObject:
            keyVaultObject["user_assigned_identity"] = None
        return keyVaultObject


class AlertAddVirtualNetwork(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core import CLIError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        VirtualNetworkList = {}
        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'id':
                VirtualNetworkList["id"] = v
            elif k == 'ignore-missing-endpoint':
                if v == "true" or v == "True" or v == "TRUE":
                    VirtualNetworkList["ignore_missing_endpoint"] = True
                else:
                    VirtualNetworkList["ignore_missing_endpoint"] = False
            else:
                raise InvalidArgumentValueError("Invalid Argument for:'{}' Only allowed arguments are 'id, ignore-missing-endpoint'".format(option_string))

        if VirtualNetworkList["id"] is None:
            raise CLIError('id is mandatory properties')

        return VirtualNetworkList


class AlertAddIpRule(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core import CLIError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        IpRuleList = {}
        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'ip-address':
                IpRuleList["ip-address"] = v
            elif k == 'action':
                IpRuleList["action"] = v
            else:
                raise InvalidArgumentValueError(
                    "Invalid Argument for:'{}' Only allowed arguments are 'ip-address, action'".format(option_string))

        if IpRuleList["ip-address"] is None:
            raise CLIError('ip-address is mandatory properties')

        if "action" not in IpRuleList:
            IpRuleList["action"] = 'Allow'
        return IpRuleList


class ConstructPolicy(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core.azclierror import RequiredArgumentMissingError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        from azure.cli.core import CLIError
        from azure.cli.command_modules.eventhubs.constants import INCOMING_MESSAGES
        from azure.cli.command_modules.eventhubs.constants import INCOMING_BYTES
        from azure.cli.command_modules.eventhubs.constants import OUTGOING_BYTES
        from azure.cli.command_modules.eventhubs.constants import OUTGOING_MESSAGES

        throttling_policy = {
            "name": None,
            "type": "ThrottlingPolicy",
            "throttling_policy": {
                "rate_limit_threshold": None,
                "metric_id": None
            }
        }

        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'name':
                throttling_policy["name"] = v

            elif k == 'rate-limit-threshold':
                if v.isdigit() is False:
                    raise CLIError('rate-limit-threshold should be an integer')
                throttling_policy["throttling_policy"]["rate_limit_threshold"] = int(v)

            elif k == 'metric-id':
                if v.lower() == INCOMING_MESSAGES.lower():
                    throttling_policy["throttling_policy"]["metric_id"] = INCOMING_MESSAGES
                elif v.lower() == INCOMING_BYTES.lower():
                    throttling_policy["throttling_policy"]["metric_id"] = INCOMING_BYTES
                elif v.lower() == OUTGOING_MESSAGES.lower():
                    throttling_policy["throttling_policy"]["metric_id"] = OUTGOING_MESSAGES
                elif v.lower() == OUTGOING_BYTES.lower():
                    throttling_policy["throttling_policy"]["metric_id"] = OUTGOING_BYTES
                else:
                    raise CLIError('Only allowed values for metric_id are: {0}, {1}, {2}, {3}'.format(INCOMING_MESSAGES, INCOMING_BYTES, OUTGOING_MESSAGES, OUTGOING_BYTES))

            else:
                raise InvalidArgumentValueError("Invalid Argument for:'{}' Only allowed arguments are 'name, rate-limit-threshold and metric-id'".format(option_string))
        if (throttling_policy["name"] is None) or (throttling_policy["throttling_policy"].get("rate_limit_threshold", None) is None) or (throttling_policy["throttling_policy"].get("metric_id", None) is None):
            raise RequiredArgumentMissingError('Throttling policies is missing one of these parameters:name, metric-id, rate-limit-threshold')

        return throttling_policy


class ConstructPolicyName(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core.azclierror import RequiredArgumentMissingError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        policy = {}
        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'name':
                policy["name"] = v
            else:
                raise InvalidArgumentValueError(
                    "Invalid Argument for:'{}' Only allowed arguments are 'name' ".format(option_string))
        if policy["name"] is None:
            raise RequiredArgumentMissingError('Throttling policies is missing the parameters: name')

        return policy


class AlertAddlocation(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.cli.core import CLIError
        from azure.cli.core.azclierror import InvalidArgumentValueError
        LocationObject = {}
        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'location-name':
                LocationObject["location_name"] = v
            elif k == 'role-type':
                LocationObject["role_type"] = v
            elif k == 'cluster-arm-id':
                LocationObject["cluster_arm_id"] = v
            else:
                raise InvalidArgumentValueError(
                    "Invalid Argument for:'{}' Only allowed arguments are 'location-name, role-type and cluster-arm-id'".format(
                        option_string))

        if (LocationObject["location_name"] is None) or (LocationObject["role_type"] is None):
            raise CLIError('location-name and role-type are mandatory properties')

        if "cluster_arm_id" not in LocationObject:
            LocationObject["cluster_arm_id"] = ''

        return LocationObject



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""
This module is vendored from
https://github.com/Azure/azure-python-devtools/tree/1.2.0/src/azure_devtools/scenario_tests

More info: https://github.com/Azure/azure-cli/pull/20601
"""

from .base import IntegrationTestBase, ReplayableTest, LiveTest
from .exceptions import AzureTestError
from .decorators import live_only, record_only, AllowLargeResponse
from .patches import mock_in_unit_test, patch_time_sleep_api, patch_long_run_operation_delay
from .preparers import AbstractPreparer, SingleValueReplacer
from .recording_processors import (
    RecordingProcessor, SubscriptionRecordingProcessor,
    LargeRequestBodyProcessor, LargeResponseBodyProcessor, LargeResponseBodyReplacer,
    OAuthRequestResponsesFilter, DeploymentNameReplacer, GeneralNameReplacer, AccessTokenReplacer, RequestUrlNormalizer,
)
from .utilities import create_random_name, get_sha1_hash

__all__ = ['IntegrationTestBase', 'ReplayableTest', 'LiveTest',
           'AzureTestError',
           'mock_in_unit_test', 'patch_time_sleep_api', 'patch_long_run_operation_delay',
           'AbstractPreparer', 'SingleValueReplacer', 'AllowLargeResponse',
           'RecordingProcessor', 'SubscriptionRecordingProcessor',
           'LargeRequestBodyProcessor', 'LargeResponseBodyProcessor', 'LargeResponseBodyReplacer',
           'OAuthRequestResponsesFilter', 'DeploymentNameReplacer', 'GeneralNameReplacer',
           'AccessTokenReplacer', 'RequestUrlNormalizer',
           'live_only', 'record_only',
           'create_random_name', 'get_sha1_hash']
__version__ = '0.5.2'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os

from time import sleep
from knack.util import CLIError
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk.scenario_tests.const import ENV_LIVE_TEST
from azure.cli.testsdk import (
    JMESPathCheck,
    NoneCheck,
    ResourceGroupPreparer,
    ScenarioTest)
from .constants import DEFAULT_LOCATION, SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH

class PostgreSQLFlexibleServerMgmtScenarioTest(ScenarioTest):

    postgres_location = DEFAULT_LOCATION

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    def test_postgres_flexible_server_mgmt(self, resource_group):
        self._test_flexible_server_mgmt(resource_group)

    def _test_flexible_server_mgmt(self, resource_group):

        if self.cli_ctx.local_context.is_on:
            self.cmd('config param-persist off')

        version = '17'
        storage_size = 128
        location = self.postgres_location
        sku_name = 'Standard_D4ads_v5'
        memory_optimized_sku = 'Standard_E4ds_v5'
        tier = 'GeneralPurpose'
        backup_retention = 7
        server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)
        zonal_resiliency_value = 'Enabled'
        ha_value = 'ZoneRedundant'

        # List SKUs
        self.cmd('postgres flexible-server list-skus -l {}'.format(location),
                 checks=[JMESPathCheck('type(@)', 'array')])

        # Create server
        self.cmd('postgres flexible-server create -g {} -n {} --backup-retention {} --sku-name {} --tier {} \
                  --storage-size {} -u {} --version {} --tags keys=3 --zonal-resiliency {} --location {}\
                  --public-access None'.format(resource_group, server_name, backup_retention,
                                               sku_name, tier, storage_size, 'dbadmin', version, zonal_resiliency_value, location))

        # Validate several properties of the server
        basic_info = self.cmd('postgres flexible-server show -g {} -n {}'.format(resource_group, server_name)).get_output_in_json()
        self.assertEqual(basic_info['name'], server_name)
        self.assertEqual(str(basic_info['location']).replace(' ', '').lower(), location)
        self.assertEqual(basic_info['resourceGroup'], resource_group)
        self.assertEqual(basic_info['sku']['name'], sku_name)
        self.assertEqual(basic_info['sku']['tier'], tier)
        self.assertEqual(basic_info['version'], version)
        self.assertEqual(basic_info['storage']['storageSizeGb'], storage_size)
        self.assertEqual(basic_info['backup']['backupRetentionDays'], backup_retention)
        self.assertEqual(basic_info['highAvailability']['mode'], ha_value)

        # List servers
        self.cmd('postgres flexible-server list -g {}'.format(resource_group),
                 checks=[JMESPathCheck('type(@)', 'array')])

        # Show connection string
        connection_string = self.cmd('postgres flexible-server show-connection-string -s {}'
                                     .format(server_name)).get_output_in_json()
        self.assertIn('jdbc', connection_string['connectionStrings'])
        self.assertIn('node.js', connection_string['connectionStrings'])
        self.assertIn('php', connection_string['connectionStrings'])
        self.assertIn('python', connection_string['connectionStrings'])
        self.assertIn('ado.net', connection_string['connectionStrings'])

        # Update password
        self.cmd('postgres flexible-server update -g {} -n {} -p randompw321##@!'
                 .format(resource_group, server_name))

        # Update compute and storage
        self.cmd('postgres flexible-server update -g {} -n {} --storage-size 256 --yes'
                 .format(resource_group, server_name),
                 checks=[JMESPathCheck('storage.storageSizeGb', 256 )])

        self.cmd('postgres flexible-server update -g {} -n {} --storage-auto-grow Enabled'
                 .format(resource_group, server_name),
                 checks=[JMESPathCheck('storage.autoGrow', "Enabled" )])

        self.cmd('postgres flexible-server update -g {} -n {} --storage-auto-grow Disabled'
                 .format(resource_group, server_name),
                 checks=[JMESPathCheck('storage.autoGrow', "Disabled" )])

        performance_tier = 'P15'
        performance_tier_lower = performance_tier.lower()

        self.cmd('postgres flexible-server update -g {} -n {} --performance-tier {}'
                 .format(resource_group, server_name, performance_tier_lower),
                 checks=[JMESPathCheck('storage.tier', performance_tier)])

        tier = 'MemoryOptimized'
        sku_name = memory_optimized_sku
        self.cmd('postgres flexible-server update -g {} -n {} --tier {} --sku-name {} --yes'
                 .format(resource_group, server_name, tier, sku_name),
                 checks=[JMESPathCheck('sku.tier', tier),
                         JMESPathCheck('sku.name', sku_name)])

        # Update backup retention
        self.cmd('postgres flexible-server update -g {} -n {} --backup-retention {}'
                 .format(resource_group, server_name, backup_retention + 10),
                 checks=[JMESPathCheck('backup.backupRetentionDays', backup_retention + 10)])
        
        # Update maintenance window
        maintainence_window = 'SUN'
        maintainence_window_value = 0   # Sunday is defined as 0
        
        self.cmd('postgres flexible-server update -g {} -n {} --maintenance-window {}'
                 .format(resource_group, server_name, maintainence_window),
                 checks=[JMESPathCheck('maintenanceWindow.dayOfWeek', maintainence_window_value)])

        # Update tags
        self.cmd('postgres flexible-server update -g {} -n {} --tags keys=3'
                 .format(resource_group, server_name),
                 checks=[JMESPathCheck('tags.keys', '3')])

        # Restart, stop, start server
        self.cmd('postgres flexible-server restart -g {} -n {}'
                 .format(resource_group, server_name), checks=NoneCheck())

        self.cmd('postgres flexible-server stop -g {} -n {}'
                 .format(resource_group, server_name), checks=NoneCheck())

        self.cmd('postgres flexible-server start -g {} -n {}'
                 .format(resource_group, server_name), checks=NoneCheck())
        
        # Negative tests
        replica_1_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)

        self.cmd('postgres flexible-server replica create -g "" --name {} --source-server {}'.format(
                        replica_1_name,
                        server_name
            ), expect_failure=True)
        self.cmd('postgres flexible-server replica create -g \'\' --name {} --source-server {}'.format(
                        replica_1_name,
                        server_name
            ), expect_failure=True)
        self.cmd('postgres flexible-server update -g "" -n {} -p randompw321##@!'
                 .format(server_name), expect_failure=True)
        self.cmd('postgres flexible-server update -g \'\' -n {} -p randompw321##@!'
                 .format(server_name), expect_failure=True)

        # Delete server
        self.cmd('postgres flexible-server delete -g {} -n {} --yes'.format(resource_group, server_name), checks=NoneCheck())
        os.environ.get(ENV_LIVE_TEST, False) and sleep(300)

        # Revive dropped server
        revived_server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)
        source_server_id = basic_info['id']
        revive_dropped_server = self.cmd('postgres flexible-server revive-dropped -g {} -n {} --source-server {} --location {}'.format(
                                         resource_group, revived_server_name, source_server_id, location)).get_output_in_json()
        self.assertEqual(revive_dropped_server['name'], revived_server_name)


class PostgreSQLFlexibleServerValidatorScenarioTest(ScenarioTest):

    postgres_location = DEFAULT_LOCATION

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    def test_postgres_flexible_server_mgmt_create_validator(self, resource_group):
        self._test_mgmt_create_validator(resource_group)

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    def test_postgres_flexible_server_mgmt_update_validator(self, resource_group):
        self._test_mgmt_update_validator(resource_group)

    def _test_mgmt_create_validator(self, resource_group):

        RANDOM_VARIABLE_MAX_LENGTH = 15
        location = self.postgres_location
        server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)
        invalid_version = self.create_random_name('version', RANDOM_VARIABLE_MAX_LENGTH)
        invalid_sku_name = self.create_random_name('sku_name', RANDOM_VARIABLE_MAX_LENGTH)
        invalid_tier = self.create_random_name('tier', RANDOM_VARIABLE_MAX_LENGTH)
        valid_tier = 'GeneralPurpose'
        invalid_backup_retention = 40
        zonal_resiliency_value = 'Enabled'
        valid_vnet_name = self.create_random_name('vnet', RANDOM_VARIABLE_MAX_LENGTH)
        invalid_vnet_name = self.create_random_name('vnet(/?\\)', RANDOM_VARIABLE_MAX_LENGTH)
        valid_subnet_name = self.create_random_name('subnet', RANDOM_VARIABLE_MAX_LENGTH)
        invalid_subnet_name = self.create_random_name('subnet(/?\\)', RANDOM_VARIABLE_MAX_LENGTH)
        valid_vnet_identifier = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/resource-group/providers/Microsoft.Network/virtualNetworks/{}'.format(valid_vnet_name)
        invalid_vnet_identifier = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/resource-group/providers/Microsoft.Network/virtualNetworks/{}'.format(invalid_vnet_name)
        valid_subnet_identifier = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/resource-group/providers/Microsoft.Network/virtualNetworks/{}/subnets/{}'.format(valid_vnet_name, valid_subnet_name)
        valid_private_dns_zone = '{}.private.postgres.database.azure.com'.format(server_name)
        invalid_private_dns_zones = ['{}.postgres.database.azure.com'.format(server_name), 'invalidprivate.dns.zone', '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/resource-group/providers/Microsoft.Network/privateDnsZones/{}.invalid(/?\\)segment.postgres.database.azure.com'.format(server_name)]

        # Create server with invalid server name.
        self.cmd('postgres flexible-server create -g {} -n Wrongserver.Name -l {}'.format(
                resource_group, location),
                expect_failure=True)

        # Create server with invalid tier.
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier {}'.format(
                 resource_group, server_name, location, invalid_tier),
                 expect_failure=True)

        # Create server with invalid version.
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --version {}'.format(
                 resource_group, server_name, location, invalid_version),
                 expect_failure=True)

        # Create server with invalid SKU name.
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier {} --sku-name {}'.format(
                 resource_group, server_name, location, valid_tier, invalid_sku_name),
                 expect_failure=True)

        # Create server with invalid backup retention days.
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --backup-retention {}'.format(
                 resource_group, server_name, location, invalid_backup_retention),
                 expect_failure=True)

        # Create server with zone redundant high availability in a location that does not support it, because it's a single zone location
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --zonal-resiliency {} '.format(
                 resource_group, server_name, location, zonal_resiliency_value),
                 expect_failure=True)

        # Create server with zone redundant high availability with a tier that does not support it
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier Burstable --sku-name Standard_B1ms --zonal-resiliency {}'.format(
                 resource_group, server_name, location, zonal_resiliency_value),
                 expect_failure=True)

        # Create server with zone redundant high availability in a location that does not support it, because it's a single zone location
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier GeneralPurpose --sku-name Standard_D4ds_v4 --zonal-resiliency {}'.format(
                 resource_group, server_name, location, zonal_resiliency_value),
                 expect_failure=True)

        # Create server with zone redundant high availability and forcing same zone for primary and standby
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier GeneralPurpose --sku-name Standard_D2ads_v5 --zonal-resiliency {} --zone 1 --standby-zone 1'.format(
                 resource_group, server_name, location, zonal_resiliency_value),
                 expect_failure=True)

        # Create server with private network arguments but without a private DNS zone.
        with self.assertRaisesRegex(CLIError, "When --vnet or --subnet is provided, --private-dns-zone is required."):
            self.cmd(
                'postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {}'.format(
                    resource_group, server_name, location, valid_vnet_name, valid_subnet_name))
        
        # Create server with public access, virtual network name, and subnet name at the same time.
        with self.assertRaisesRegex(CLIError, "The --subnet and --public-access arguments cannot be used together. Use only one of them."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --public-access All'.format(
                    resource_group, server_name, location, valid_vnet_name, valid_subnet_name))

        # Create server with an incorrectly formed virtual network identifier.
        with self.assertRaisesRegex(CLIError, "Specify either --subnet as a subnet resource identifier, --vnet as a vnet resource identifier with --subnet as a subnet name, or both --vnet and --subnet as resource names."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, invalid_vnet_identifier, valid_subnet_name, valid_private_dns_zone))

        # Create server with an incorrectly formed subnet name.
        with self.assertRaisesRegex(CLIError, "Specify either --subnet as a subnet resource identifier, --vnet as a vnet resource identifier with --subnet as a subnet name, or both --vnet and --subnet as resource names."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_vnet_name, invalid_subnet_name, valid_private_dns_zone))

        # Create server with an incorrectly formed virtual network name.
        with self.assertRaisesRegex(CLIError, "Specify either --subnet as a subnet resource identifier, --vnet as a vnet resource identifier with --subnet as a subnet name, or both --vnet and --subnet as resource names."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, invalid_vnet_name, valid_subnet_name, valid_private_dns_zone))

        # Create server with the resource identifier of a virtual network and the resource identifier of a subnet at the same time.
        with self.assertRaisesRegex(CLIError, "If you pass --vnet as a resource identifier, --subnet must be a subnet name."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_vnet_identifier, valid_subnet_identifier, valid_private_dns_zone))

        # Create server with the resource identifier of a subnet, and the name of a virtual network at the same time.
        with self.assertRaisesRegex(CLIError, "Specify either --subnet as a subnet resource identifier, --vnet as a vnet resource identifier with --subnet as a subnet name, or both --vnet and --subnet as resource names."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_vnet_name, valid_subnet_identifier, valid_private_dns_zone))

        # Create server with a private DNS zone, but without virtual network and subnet.
        with self.assertRaisesRegex(CLIError, "Private DNS zone can only be used with private access network. Use --vnet or/and --subnet parameters."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_private_dns_zone))

        # Create server with a private DNS zone and a virtual network name, but without a subnet name.
        with self.assertRaisesRegex(CLIError, "When --vnet is provided, --subnet must also be provided in the form of subnet name."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_vnet_name, valid_private_dns_zone))

        # Create server with a private DNS zone and a subnet name, but without a virtual network.
        with self.assertRaisesRegex(CLIError, "Incorrectly formed subnet identifier. If you are providing only --subnet but not --vnet, the --subnet parameter should be in resource identifier format."):
            self.cmd('postgres flexible-server create -g {} -n {} -l {} --subnet {} --private-dns-zone {}'.format(
                    resource_group, server_name, location, valid_subnet_name, valid_private_dns_zone))

        # Create server with multiple forms of invalid private DNS zones.
        messages = [
            "The private DNS zone name cannot be the same as the server's fully qualified domain name.",
            "The suffix of the private DNS zone should be '.postgres.database.azure.com'.",
            "The private DNS zone name or identifier is not in a valid format."
        ]
        for invalid_private_dns_zone in invalid_private_dns_zones:
            with self.assertRaisesRegex(CLIError, messages[invalid_private_dns_zones.index(invalid_private_dns_zone)]):
                self.cmd('postgres flexible-server create -g {} -n {} -l {} --vnet {} --subnet {} --private-dns-zone {}'.format(
                        resource_group, server_name, location, valid_vnet_name, valid_subnet_name, invalid_private_dns_zone))

        # Create server with invalid public access value (end-ip is not a valid IPv4 address).
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --public-access 12.0.0.0-10.0.0.0.0'.format(
                 resource_group, server_name, location),
                 expect_failure=True)

        # Create server with invalid public access value (start-ip greater than end-ip).
        self.cmd('postgres flexible-server create -g {} -n {} -l {} --public-access 12.0.0.0-10.0.0.0'.format(
                 resource_group, server_name, location),
                 expect_failure=True)

        # Create server with invalid storage size (in Premium_LRS, valid storage sizes are 32, 64, 128, 256, 512, 1024, 2048, 4095, 4096, 8192, 16384, 32768, 65536).
        invalid_storage_size = 60
        self.cmd('postgres flexible-server create -g {} -l {} --storage-size {} --public-access none'.format(
                 resource_group, location, invalid_storage_size),
                 expect_failure=True)

    def _test_mgmt_update_validator(self, resource_group):
        RANDOM_VARIABLE_MAX_LENGTH = 30
        server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)
        invalid_sku_name = self.create_random_name('sku_name', RANDOM_VARIABLE_MAX_LENGTH)
        invalid_tier = self.create_random_name('tier', RANDOM_VARIABLE_MAX_LENGTH)
        valid_tier = 'GeneralPurpose'
        invalid_backup_retention = 40
        version = 16
        storage_size = 128
        location = self.postgres_location
        tier = 'Burstable'
        sku_name = 'Standard_B1ms'
        backup_retention = 10

        list_checks = [JMESPathCheck('name', server_name),
                       JMESPathCheck('resourceGroup', resource_group),
                       JMESPathCheck('sku.name', sku_name),
                       JMESPathCheck('sku.tier', tier),
                       JMESPathCheck('version', version),
                       JMESPathCheck('storage.storageSizeGb', storage_size),
                       JMESPathCheck('backup.backupRetentionDays', backup_retention)]

        self.cmd('postgres flexible-server create -g {} -n {} -l {} --tier {} --version {} --sku-name {} --storage-size {} --backup-retention {} --public-access none'
                 .format(resource_group, server_name, location, tier, version, sku_name, storage_size, backup_retention))
        self.cmd('postgres flexible-server show -g {} -n {}'.format(resource_group, server_name), checks=list_checks)

        # Update server with invalid tier for current sku
        invalid_tier = 'GeneralPurpose'
        self.cmd('postgres flexible-server update -g {} -n {} --tier {}'.format(
                 resource_group, server_name, invalid_tier),
                 expect_failure=True)

        # Update server with invalid sku for the given tier
        self.cmd('postgres flexible-server update -g {} -n {} --tier {} --sku-name {}'.format(
                 resource_group, server_name, valid_tier, invalid_sku_name),
                 expect_failure=True)

        # Update server with smaller than current storage size
        invalid_storage_size = 64
        self.cmd('postgres flexible-server update -g {} -n {} --storage-size {}'.format(
                 resource_group, server_name, invalid_storage_size),
                 expect_failure=True)

        # Update server with invalid backup retention days
        self.cmd('postgres flexible-server update -g {} -n {} --backup-retention {}'.format(
                 resource_group, server_name, invalid_backup_retention),
                 expect_failure=True)

        # Update server with high availability on a tier that does not support it
        zonal_resiliency_value = 'Enabled'
        self.cmd('postgres flexible-server update -g {} -n {} --zonal-resiliency {}'.format(
                 resource_group, server_name, zonal_resiliency_value),
                 expect_failure=True)

        # Delete server
        self.cmd('postgres flexible-server delete -g {} -n {} --yes'.format(
                 resource_group, server_name), checks=NoneCheck())



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: disable=too-many-lines
# pylint: disable=too-many-statements, protected-access

from knack.log import get_logger

from .aaz.latest.databoxedge.order import Create as _OrderCreate


logger = get_logger(__name__)


class OrderCreate(_OrderCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        schema = super()._build_arguments_schema(*args, **kwargs)
        schema.company_name._required = True
        schema.contact_person._required = True
        schema.email_list._required = True
        schema.phone._required = True
        schema.status._required = True
        schema.address_line1._required = True
        schema.city._required = True
        schema.country._required = True
        schema.postal_code._required = True
        schema.state._required = True
        return schema



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-few-public-methods

import unittest
import os
import re
from pytest import skip
import requests
from knack.util import CLIError

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (
    ScenarioTest, ResourceGroupPreparer, JMESPathCheck, live_only)

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
WINDOWS_ASP_LOCATION_WEBAPP = 'japanwest'
LINUX_ASP_LOCATION_WEBAPP = 'eastus2'


class WebAppUpE2ETests(ScenarioTest):
    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_no_plan_e2e(self, resource_group):
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test the full E2E operation works
        self.cmd('webapp up -n {} -g {}'.format(webapp_name, resource_group)).get_output_in_json()

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        config = self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
        ]).get_output_in_json()

        self.assertIn("node", config.get("linuxFxVersion", "").lower())

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)


    @live_only()
    @unittest.skip("Flaky test skipping")
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_no_plan_different_os_e2e(self, resource_group):
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test the full E2E operation works
        self.cmd('webapp up -n {} -g {} --os-type windows'.format(webapp_name, resource_group))

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        webapp_name = self.create_random_name('up-nodeapp', 24)
        self.cmd('webapp up -n {} -g {} --os-type linux'.format(webapp_name, resource_group), expect_failure=True)

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)


    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_node_e2e(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} --dryrun'.format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'premiumv2')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertIn("node|", result['runtime_version'].lower())
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        config = self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
        ]).get_output_in_json()
        self.assertIn("node", config.get("linuxFxVersion", "").lower())

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', True),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'PremiumV2'),
            JMESPathCheck('sku.name', 'P1v2')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_python_e2e(self, resource_group):
        plan = self.create_random_name('up-pythonplan', 24)
        webapp_name = self.create_random_name('up-pythonapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'python-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'python-docs-hello-world')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd('webapp up -n {} --sku  S1 --dryrun'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'python|3.7')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} --sku  S1 -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('linuxFxVersion', 'PYTHON|3.7'),
            JMESPathCheck('tags.cli', 'None'),
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', True),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Standard'),
            JMESPathCheck('sku.name', 'S1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_dotnetcore_e2e(self, resource_group):
        plan = self.create_random_name('up-dotnetcoreplan', 24)
        webapp_name = self.create_random_name('up-dotnetcoreapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'dotnetcore-hello-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'hellodotnetcore')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd('webapp up -n {} --dryrun'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'free')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'dotnetcore|3.1')
        self.assertEqual(result['os'].lower(), 'windows')
        self.assertNotEqual(result['location'], 'None')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])
        self.assertEqual(result['location'], full_result['location'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
            JMESPathCheck('windowsFxVersion', None)
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', False),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Free'),
            JMESPathCheck('sku.name', 'F1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_dotnet6_e2e(self, resource_group):
        plan = self.create_random_name('up-dotnetplan', 24)
        webapp_name = self.create_random_name('up-dotnetapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'dotnet6-hello-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = temp_dir
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd('webapp up -n {} --dryrun --os-type Linux'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'free')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'dotnetcore|6.0')
        self.assertEqual(result['os'].lower(), 'linux')
        self.assertNotEqual(result['location'], 'None')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])
        self.assertEqual(result['location'], full_result['location'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
            JMESPathCheck('windowsFxVersion', None)
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', False),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Free'),
            JMESPathCheck('sku.name', 'F1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_nested_dotnet_project_e2e(self, resource_group):
        plan = self.create_random_name('up-dotnetplan', 24)
        webapp_name = self.create_random_name('up-dotnetapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'dotnet6-nested-hello-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = temp_dir
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd('webapp up -n {} --dryrun --os-type Linux'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'free')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'], 'dotnetcore|6.0')
        self.assertEqual(result['os'].lower(), 'linux')
        self.assertNotEqual(result['location'], 'None')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])
        self.assertEqual(result['location'], full_result['location'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
            JMESPathCheck('windowsFxVersion', None)
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', False),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Free'),
            JMESPathCheck('sku.name', 'F1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_dotnet6_e2e(self, resource_group):
        plan = self.create_random_name('up-dotnetplan', 24)
        webapp_name = self.create_random_name('up-dotnetapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'dotnet6-webapi-with-dependencies-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the WebApplication has been extracted to.
        # the zip contains more than one projects, it is assumed that the user is in this directory
        # when he makes the webapp up.
        up_working_dir = temp_dir
        os.chdir(os.path.join(up_working_dir, "dotnet6-webapi-with-dependencies-up"))

        # test dryrun operation
        result = self.cmd('webapp up -n {} --dryrun --os-type Linux'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'free')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'dotnetcore|6.0')
        self.assertEqual(result['os'].lower(), 'linux')
        self.assertNotEqual(result['location'], 'None')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])
        self.assertEqual(result['location'], full_result['location'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
            JMESPathCheck('windowsFxVersion', None)
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', False),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Free'),
            JMESPathCheck('sku.name', 'F1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_statichtml_e2e(self, resource_group):
        plan = self.create_random_name('up-statichtmlplan', 24)
        webapp_name = self.create_random_name('up-statichtmlapp', 24)
        zip_file_name = os.path.join(
            TEST_DIR, 'html-static-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'html-docs-hello-world-master')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd('webapp up -n {} --dryrun --html'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'free')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), '-')
        self.assertEqual(result['os'].lower(), 'windows')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --html'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None'),
            JMESPathCheck('windowsFxVersion', None)
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[1].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[1].value', 'True')
        ])

        # verify SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', False),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Free'),
            JMESPathCheck('sku.name', 'F1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_invalid_name(self, resource_group):
        webapp_name = self.create_random_name('invalid_name', 40)
        zip_file_name = os.path.join(TEST_DIR, 'python-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'python-docs-hello-world')
        os.chdir(up_working_dir)

        from azure.cli.core.util import CLIError
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {} --dryrun'.format(webapp_name))
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {}'.format(webapp_name))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_name_exists_not_in_subscription(self, resource_group):
        # Make sure webapp_name is the name of an existing web app and is not in your subscription
        webapp_name = 'helloworld'
        zip_file_name = os.path.join(TEST_DIR, 'python-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'python-docs-hello-world')
        os.chdir(up_working_dir)

        from azure.cli.core.util import CLIError
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {} --dryrun'.format(webapp_name))
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {}'.format(webapp_name))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_name_exists_in_subscription(self, resource_group):
        plan = self.create_random_name('up-name-exists-plan', 40)
        webapp_name = self.create_random_name('up-name-exists-app', 40)
        zip_file_name = os.path.join(TEST_DIR, 'python-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'python-docs-hello-world')
        os.chdir(up_working_dir)

        # create a webapp with the same name
        self.cmd(
            'appservice plan create -g {} -n {} --sku S1 --is-linux'.format(resource_group, plan))
        self.cmd(
            'webapp create -g {} -n {} --plan {} -r "python|3.7"'.format(resource_group, webapp_name, plan))
        requests.get('http://{}.azurewebsites.net'.format(webapp_name), timeout=240)
        self.cmd('webapp up -n {}'.format(webapp_name))
        self.cmd('webapp list -g {}'.format(resource_group), checks=[
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].name', webapp_name),
            JMESPathCheck('[0].hostNames[0]', webapp_name +
                          '.azurewebsites.net')
        ])

        # test dryrun operation
        result = self.cmd('webapp up -n {} --sku S1 --dryrun'
                          .format(webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'], 'python|3.7')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} --sku S1 -g {} --plan {}'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_choose_os(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os-type "linux" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'premiumv2')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertIn("node|", result['runtime_version'].lower())
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os-type "linux"'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None')
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        self.cmd('appservice plan show', checks=[
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'PremiumV2'),
            JMESPathCheck('sku.name', 'P1v2')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_choose_runtime(self, resource_group):
        plan = self.create_random_name('up-pythonplan', 24)
        webapp_name = self.create_random_name('up-pythonapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'python-hello-world-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'python-docs-hello-world')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --runtime "PYTHON|3.7" --sku S1 --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'python|3.7')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --runtime "PYTHON|3.7" --sku "S1"'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('linuxFxVersion', 'PYTHON|3.7'),
            JMESPathCheck('tags.cli', 'None')
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', True),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Standard'),
            JMESPathCheck('sku.name', 'S1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_choose_os_and_runtime(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "linux" --runtime "node|22-lts" --sku "S1" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22-lts')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "linux" --runtime "node|22-lts" --sku "S1"'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('tags.cli', 'None')
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        self.cmd('appservice plan show', checks=[
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'Standard'),
            JMESPathCheck('sku.name', 'S1')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_runtime_delimiters(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "windows" --runtime "java|1.8|Java SE|8" --sku "S1" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'java|1.8|java se|8')

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "windows" --runtime "java:1.8:Java SE:8" --sku "S1" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'java|1.8|java se|8')

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_linux_to_windows_fail(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "linux" --runtime "node|22-lts" --sku "S1" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22-lts')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "linux" --runtime "node|22-lts" --sku "S1"'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        from azure.cli.core.util import CLIError
        # changing existing linux app to windows should fail gracefully
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {} -g {} --plan {} --os "windows" --runtime "node|22LTS" --sku "S1"'.format(webapp_name, resource_group, plan))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @unittest.skip("Temp skip - flaky test")
    def test_windows_to_linux_fail(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up-windows.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(temp_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "windows" --runtime "node|22LTS" --sku "S1" --dryrun'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22lts')
        self.assertEqual(result['os'].lower(), 'windows')

        # test the full E2E operation works
        full_result = self.cmd(
            'webapp up -n {} -g {} --plan {} --os "windows" --runtime "node|22LTS" --sku "S1"'.format(webapp_name, resource_group, plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        from azure.cli.core.util import CLIError
        # changing existing linux app to windows should fail gracefully
        with self.assertRaises(CLIError):
            self.cmd('webapp up -n {} -g {} --plan {} --os "linux" --runtime "node|22-LTS" --sku "S1"'.format(webapp_name, resource_group, plan))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_webapp_up_runtime_case_insensitivity_windows(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        webapp_name = self.create_random_name('up-nodeapp', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node_app.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        os.chdir(temp_dir)

        # test the full E2E operation works
        self.cmd('webapp up -n {} -g {} --plan {} --os "windows" --runtime "node|22-LTS"'.format(webapp_name, resource_group, plan))

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('kind', 'app'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        # verify runtime version
        app_settings = self.cmd('webapp config appsettings list').get_output_in_json()
        app_settings = {s["name"] : s["value"] for s in app_settings}
        self.assertIn("WEBSITE_NODE_DEFAULT_VERSION", app_settings)
        self.assertEqual(app_settings["WEBSITE_NODE_DEFAULT_VERSION"], "~22")

        # re-run with a differently-cased runtime alias to verify idempotency / case-insensitivity
        # (Windows App Service currently only exposes Node 22 LTS, so we cannot switch to another
        # Node major; this exercise confirms 'webapp up' treats runtime tokens case-insensitively.)
        self.cmd('webapp up -n {} -g {} --plan {} --os "windows" --runtime "node|22-lts"'.format(webapp_name, resource_group, plan))

        # verify the runtime is unchanged
        app_settings = self.cmd('webapp config appsettings list').get_output_in_json()
        app_settings = {s["name"] : s["value"] for s in app_settings}
        self.assertIn("WEBSITE_NODE_DEFAULT_VERSION", app_settings)
        self.assertEqual(app_settings["WEBSITE_NODE_DEFAULT_VERSION"], "~22")

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_generate_default_name(self, resource_group):
        plan = self.create_random_name('up-nodeplan', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun operation
        result = self.cmd(
            'webapp up --dryrun').get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'premiumv2')
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22-lts')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        self.cmd(
            'webapp up -g {} --plan {}'.format(resource_group, plan)).get_output_in_json()

        # Verify app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('linuxFxVersion', 'NODE|22-lts'),
            JMESPathCheck('tags.cli', 'None'),
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', True),
            JMESPathCheck('name', plan),
            JMESPathCheck('sku.tier', 'PremiumV2'),
            JMESPathCheck('sku.name', 'P1v2')
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)


    @live_only()
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_linux_windows_sharing_resource_group(self, resource_group):
        linux_plan = self.create_random_name('plan-linux', 24)
        linux_webapp_name = self.create_random_name('app-linux', 24)
        windows_plan = self.create_random_name('plan-windows', 26)
        windows_webapp_name = self.create_random_name('app-windows', 26)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test linux dryrun operation
        result = self.cmd('webapp up -n {} --sku  S1 --dryrun'
                          .format(linux_webapp_name)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(linux_webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22-lts')
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full linux E2E operation works
        full_result = self.cmd(
            'webapp up -n {} --sku  S1 -g {} --plan {} --os-type linux'.format(linux_webapp_name, resource_group, linux_plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify linux app is created
        # since we set local context, -n and -g are no longer required
        self.cmd('webapp show', checks=[
            JMESPathCheck('name', linux_webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('kind', 'app,linux'),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        self.cmd('webapp config show', checks=[
            JMESPathCheck('linuxFxVersion', 'NODE|22-lts'),
            JMESPathCheck('tags.cli', 'None'),
        ])

        self.cmd('webapp config appsettings list', checks=[
            JMESPathCheck('[0].name', 'SCM_DO_BUILD_DURING_DEPLOYMENT'),
            JMESPathCheck('[0].value', 'True')
        ])

        # test windows dryrun operation
        result = self.cmd("webapp up -n {} --sku  S1 --dryrun -r 'node|22lts' --os-type windows --plan {}"
                          .format(windows_webapp_name, windows_plan)).get_output_in_json()
        self.assertEqual(result['sku'].lower(), 'standard')
        self.assertTrue(result['name'].startswith(windows_webapp_name))
        self.assertTrue(result['src_path'].replace(
            os.sep + os.sep, os.sep), up_working_dir)
        self.assertEqual(result['runtime_version'].lower(), 'node|22lts')
        self.assertEqual(result['os'].lower(), 'windows')

        # test the full windows E2E operation works
        full_result = self.cmd(
            'webapp up -n {} --sku  S1 -g {} --plan {}'.format(windows_webapp_name, resource_group, windows_plan)).get_output_in_json()
        self.assertEqual(result['name'], full_result['name'])

        # Verify windows app is created
        self.cmd('webapp show -g {} -n {}'.format(resource_group, windows_webapp_name), checks=[
            JMESPathCheck('name', windows_webapp_name),
            JMESPathCheck('httpsOnly', True),
            JMESPathCheck('resourceGroup', resource_group)
        ])

        # verify windows SKU and kind of ASP created
        self.cmd('appservice plan show', checks=[
            JMESPathCheck('properties.reserved', True),
            JMESPathCheck('name', windows_plan),
            JMESPathCheck('sku.tier', 'Standard'),
            JMESPathCheck('sku.name', 'S1')
        ])


        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(name_prefix='cli_test_webapp_up_runtimestatus', location='eastus')
    def test_webapp_up_track_runtimestatus_runtimesuccessful(self, resource_group):
        plan_name = self.create_random_name('plan-linux', 24)
        webapp_name = self.create_random_name('app-linux', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        self.cmd('webapp up -g {} -n {} --os-type linux'.format(resource_group, webapp_name)).assert_with_checks([
            JMESPathCheck('resourcegroup', resource_group),
            JMESPathCheck('name', webapp_name),
        ])

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(name_prefix='cli_test_webapp_up_runtimestatus', location='eastus')
    def test_webapp_up_track_runtimestatus_buildfailed(self, resource_group):
        plan_name = self.create_random_name('plan-linux', 24)
        webapp_name = self.create_random_name('app-linux', 24)
        zip_file_name = os.path.join(TEST_DIR, 'data', 'nodebuildfailed.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        os.chdir(temp_dir)

        with self.assertRaisesRegex(CLIError, "Deployment failed because the build process failed"):
            self.cmd('webapp up -g {} -n {} --os-type linux -r "NODE|22-LTS"'.format(resource_group, webapp_name))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @ResourceGroupPreparer(name_prefix='cli_test_webapp_up_runtimestatus', location='eastus')
    def test_webapp_up_track_runtimestatus_runtimefailed(self, resource_group):
        plan_name = self.create_random_name('plan-linux', 24)
        webapp_name = self.create_random_name('app-linux', 24)
        zip_file_name = os.path.join(TEST_DIR, 'data', 'noderuntimefailed.zip')

        # create a temp directory and unzip the code to this folder
        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        # change the working dir to the dir where the code has been extracted to
        os.chdir(temp_dir)

        with self.assertRaisesRegex(CLIError, "Deployment failed because the site failed to start within 10 mins."):
            self.cmd('webapp up -g {} -n {} --os-type linux -r "NODE|22-LTS"'.format(resource_group, webapp_name))

        # cleanup
        # switch back the working dir
        os.chdir(current_working_dir)
        # delete temp_dir
        import shutil
        shutil.rmtree(temp_dir)

    @live_only()
    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(random_name_length=24, name_prefix='clitest', location=LINUX_ASP_LOCATION_WEBAPP)
    def test_webapp_up_with_domain_name_scope(self, resource_group):
        plan = self.create_random_name('up-dnlplan', 24)
        webapp_name = self.create_random_name('up-dnl-app', 24)
        zip_file_name = os.path.join(TEST_DIR, 'node-Express-up.zip')

        import zipfile
        import tempfile
        temp_dir = tempfile.mkdtemp()
        zip_ref = zipfile.ZipFile(zip_file_name, 'r')
        zip_ref.extractall(temp_dir)
        current_working_dir = os.getcwd()

        up_working_dir = os.path.join(temp_dir, 'myExpressApp')
        os.chdir(up_working_dir)

        # test dryrun with --domain-name-scope
        result = self.cmd('webapp up -n {} --dryrun --domain-name-scope TenantReuse'.format(
            webapp_name)).get_output_in_json()
        self.assertTrue(result['name'].startswith(webapp_name))
        self.assertIn("node|", result['runtime_version'].lower())
        self.assertEqual(result['os'].lower(), 'linux')

        # test the full E2E operation works
        self.cmd('webapp up -n {} -g {} --plan {} --domain-name-scope TenantReuse'.format(
            webapp_name, resource_group, plan)).get_output_in_json()

        # Verify app is created with domain name scope
        result = self.cmd('webapp show -n {} -g {}'.format(webapp_name, resource_group), checks=[
            JMESPathCheck('name', webapp_name),
            JMESPathCheck('state', 'Running'),
            JMESPathCheck('resourceGroup', resource_group),
            JMESPathCheck('autoGeneratedDomainNameLabelScope', 'TenantReuse')
        ]).get_output_in_json()

        # Verify the default hostname matches the regional pattern with hash
        default_hostname = result.get('defaultHostName')
        pattern = r'^([a-zA-Z0-9\-]+)-([a-z0-9]{16})\.([a-z]+-\d{2})\.azurewebsites\.net$'
        match = re.match(pattern, default_hostname)
        self.assertIsNotNone(match, "defaultHostName '{}' does not match expected pattern".format(default_hostname))
        app_name, hash_part, region = match.groups()
        self.assertTrue(len(hash_part) == 16 and hash_part.islower(), "Hash is not 16 chars or not lowercase.")
        self.assertIn('-', region, "Region part does not have '-' separator.")
        self.assertEqual(app_name, webapp_name, "App name and defaultHostName app name do not match.")

        # cleanup
        os.chdir(current_working_dir)
        import shutil
        shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import datetime
import json
import logging
import os

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
logger.addHandler(ch)

BUILD_ID = os.environ.get('BUILD_ID', None)
BUILD_BRANCH = os.environ.get('BUILD_BRANCH', None)

def load_module_stats():
    stats_dir = "/tmp/module_stats"
    all_stats = {}

    with open("/mnt/vss/_work/1/s/scripts/ci/core_modules.txt", "r") as f:
        core_modules = [line.strip() for line in f.readlines()]

    with open("/mnt/vss/_work/1/s/scripts/ci/extension_modules.txt", "r") as f:
        extension_modules = [line.strip() for line in f.readlines()]

    for module in core_modules + extension_modules:
        stats_file = os.path.join(stats_dir, f"{module}.json")
        if os.path.exists(stats_file):
            with open(stats_file, "r") as f:
                try:
                    stats = json.load(f)
                    codegenV1 = stats.get("codegenV1", 0)
                    codegenV2 = stats.get("codegenV2", 0)
                    total = stats.get("total", 0)
                    manual = total - codegenV1 - codegenV2
                    all_stats[module] = {
                        "codegenV1": codegenV1,
                        "codegenV2": codegenV2,
                        "manual": manual,
                        "total": total,
                        "type": "core" if module in core_modules else "extension"
                    }
                except json.JSONDecodeError:
                    logger.info(f"Warning: Could not parse {stats_file}")
    return all_stats

def analyze_stats(all_stats):
    counters = {
        "manual": {"core": 0, "extension": 0},
        "mixed": {"core": 0, "extension": 0},
        "codegen": {"core": 0, "extension": 0},
        "codegenV1": {"core": 0, "extension": 0},
        "total": {"core": 0, "extension": 0}
    }
    for _, stats in all_stats.items():
        module_type = stats["type"]
        counters["total"][module_type] += 1
        if stats["manual"] > 0 and (stats["codegenV1"] > 0 or stats["codegenV2"] > 0):
            counters["mixed"][module_type] += 1
        if stats["codegenV1"] > 0 or stats["codegenV2"] > 0:
            counters["codegen"][module_type] += 1
        if stats["codegenV1"] > 0:
            counters["codegenV1"][module_type] += 1
    counters["manual"]["core"] = counters["total"]["core"] - counters["codegen"]["core"]
    counters["manual"]["extension"] = counters["total"]["extension"] - counters["codegen"]["extension"]
    return counters

def print_results(counters):
    logger.info("\n===== Codegen Coverage Report =====")
    logger.info("\n1. Manual Modules:")
    logger.info(f"   Core: {counters['manual']['core']}")
    logger.info(f"   Extension: {counters['manual']['extension']}")
    logger.info("\n2. Mixed Modules:")
    logger.info(f"   Core: {counters['mixed']['core']}")
    logger.info(f"   Extension: {counters['mixed']['extension']}")
    logger.info("\n3. Codegen Modules:")
    logger.info(f"   Core: {counters['codegen']['core']}")
    logger.info(f"   Extension: {counters['codegen']['extension']}")
    logger.info("\n4. CodegenV1 Modules:")
    logger.info(f"   Core: {counters['codegenV1']['core']}")
    logger.info(f"   Extension: {counters['codegenV1']['extension']}")
    logger.info("\n5. Total Modules:")
    logger.info(f"   Core: {counters['total']['core']}")
    logger.info(f"   Extension: {counters['total']['extension']}")

def analyze_and_report():
    all_stats = load_module_stats()
    counters = analyze_stats(all_stats)
    print_results(counters)
    output = {
        "detailed_stats": all_stats,
        "summary": counters
    }
    logger.info("\n=== Detailed JSON Output ===")
    logger.info(json.dumps(output, indent=2))
    with open("/tmp/module_stats_summary.json", "w") as f:
        json.dump(output, f, indent=2)

def generate_csv():
    data = []
    with open(f'/tmp/codegen_report.json', 'r') as file:
        ref = json.load(file)
    codegenv1 = ref['codegenV1']
    codegenv2 = ref['codegenV2']
    total = ref['total']
    manual = total - codegenv1 - codegenv2
    is_release = True if BUILD_BRANCH == 'release' else False
    date = (datetime.datetime.utcnow() + datetime.timedelta(hours=8)).strftime("%Y-%m-%d")
    data.append([BUILD_ID, manual, codegenv1, codegenv2, total, is_release, date])
    logger.info('Finish generate data for codegen report:')
    logger.info("BUILD_ID, manual, codegenv1, codegenv2, total, is_release, date")
    logger.info(f'{data}')
    return data

if __name__ == '__main__':
    analyze_and_report()
    generate_csv()
